
# an implementation of:
# http://standards.freedesktop.org/icon-theme-spec/icon-theme-spec-latest.html
# (or at least, enough of it to do mime_type->icon file conversion; more could 
#  be added later if someone was keen..)

from ConfigParser import ConfigParser
import xdgbasedir
import os

class IconTheme:
    def __init__(self, icon_theme):
        self.icon_theme = icon_theme
        # list of directories in order, in which to look for files
        self.locs = []
        home = os.getenv('HOME')
        if home:
            self.locs.append(os.path.join(home, '.icons'))
        for path in xdgbasedir.xdg_data_dirs():
            self.locs.append(os.path.join(path, 'icons'))
        self.locs.append('/usr/share/pixmaps')
        self.locs = filter(lambda x: os.access(x, os.R_OK), self.locs)

        # find our index.theme file
        index_theme_file = None
        for path in self.locs:
            ini = os.path.join(path, icon_theme, 'index.theme')
            if os.access(ini, os.R_OK):
                index_theme_file = ini
                break
        
        if not index_theme_file:
            raise Exception("Unable to load index.theme for theme %s" % icon_theme)
        
        self.cp = cp = ConfigParser()
        cp.read(index_theme_file)
        it = "Icon Theme"
        if not cp.has_section(it):
            raise Exception("Theme does not have an 'Icon Theme' section.")
        if not cp.has_option(it, "Directories"):
            raise Exception("Theme does not specify its directories.")
        else:
            self.directories = cp.get(it, "Directories").split(',')
        if cp.has_option(it, "Inherits"):
            self.inherits = cp.get(it, "Inherits").split(',')
        elif icon_theme != 'hicolor':
            self.inherits = ['hicolor']
        else:
            self.inherits = []
        self.inherits = map(IconTheme, self.inherits)

        # most of the time, we'll care about directories by type and 
        # then by size
        self.dir_by_size = {}
        for path in self.directories:
            context = cp.get(path, 'Context', None)
            if not cp.has_option(path, 'Size'):
                raise Exception("Directory '%s' has no size." % (path))
            size = cp.get(path, 'Size')
            self.dir_by_size.setdefault(context, {}).setdefault(size, []).append(path)

    def lookup(self, icon_name, contexts=None, size=None, accept_extensions=None):
        if accept_extensions == None:
            accept_extensions = ["png", "xpm", "svg"]
        to_scan = []
        if contexts == None:
            contexts = self.dir_by_size.keys()

        for context in contexts:
            if not self.dir_by_size.has_key(context):
                continue
            for dir_size in self.dir_by_size[context]:
                if size == None or size == dir_size:
                    for path in self.dir_by_size[context][dir_size]:
                        for loc in self.locs:
                            for ext in accept_extensions:
                                attempt = os.path.join(loc, self.icon_theme, path, icon_name+'.'+ext)
                                #print "attempt:", attempt
                                if os.access(attempt, os.R_OK):
                                    #print "success!"
                                    return attempt
        
        # if we get here, try our parent themes
        for it in self.inherits:
            rv = it.lookup(icon_name, contexts, size, accept_extensions)
            if rv: return rv
        return None

class MimeIcon:
    def __init__(self, icon_theme, size=None):
        self.icon_theme = icon_theme
        self.size = size
        self.cache = {}
    
    def lookup(self, mime_type):
        def __lookup():
            def gnome_mime(mime_type):
                return "gnome-mime-" + mime_type.replace('/', '-')
            # iff we are a inode/ type, then let's look in 'Places' instead
            if mime_type.startswith('inode/'):
                gnome_name = 'gnome-fs-' + mime_type.split('/')[-1]
                rv = self.icon_theme.lookup(gnome_name, contexts=['Places', 'FileSystems'], size=self.size)
                if rv: return rv
            # stolen from gnome-ui; it's rather unfortunate but there is not a standard 
            # on how to name mime icons!
            # http://cvs.gnome.org/viewcvs/libgnomeui/libgnomeui/gnome-icon-lookup.c
            gnome_name = gnome_mime(mime_type)
            rv = self.icon_theme.lookup(gnome_name, contexts=['MimeTypes'], size=self.size)
            if rv: return rv
            # try the x-generic stuff
            generic_name = mime_type.split('/')[0] + '-x-generic'
            rv = self.icon_theme.lookup(generic_name, contexts=['MimeTypes'], size=self.size)
            if rv: return rv
            # otherwise, this seems to work some of the time
            gnome_name = gnome_mime(mime_type.split('/')[0])
            rv = self.icon_theme.lookup(gnome_name, contexts=['MimeTypes'], size=self.size)
            if rv: return rv
            # otherwise, otherwise, one of these should work\n
            really_fallbacks = [(['Applications'], 'gnome-unknown'), (['MimeTypes'], 'unknown')]
            for contexts, icon_name in really_fallbacks:
                rv = self.icon_theme.lookup(icon_name, contexts=contexts, size=self.size)
                if rv: return rv
        if not self.cache.has_key(mime_type):
            self.cache[mime_type] = __lookup()
        return self.cache[mime_type]

if __name__ == '__main__':
    it = IconTheme('gnome')
    mi = MimeIcon(it, size="16")
    print mi.lookup('inode/directory')

###
### vi:expandtab:sw=4:ts=4
###
